#include <ell/CLatticeProteinPivotMoveState.hh>
#include <fstream>	// for readBinaryInputClone(..), writeBinaryOutput(..)

namespace ell
{
	CLatticeProteinPivotMoveState::CLatticeProteinPivotMoveState(
	 const LatticeModel* _lattice, const ContactEnergyFunction* _energy,
	 const Sequence& seq, const Structure& struc)
	 : seqLength(seq.size()), lattice(_lattice), energy(_energy),
	   cSequence(compressSeq(seq, energy->getAlphabet())),
	   cStructure(compressSeq(struc, lattice->getDescriptor()->getAlphabet()))
	{}

	CLatticeProteinPivotMoveState::~CLatticeProteinPivotMoveState() {
	}

	std::string
	CLatticeProteinPivotMoveState::getUncompressedSeqStr() const {
		return uncompressSeq(cSequence, energy->getAlphabet(),
							   seqLength);
	}

	std::string
	CLatticeProteinPivotMoveState::getUncompressedStructStr() const {
			// structure string holds one character less than sequence string
		return uncompressSeq(cStructure, lattice->getDescriptor()
							   ->getAlphabet(), seqLength-1);
	}

	bool	
	CLatticeProteinPivotMoveState::operator== (const CState& _cState2) const {
		CLatticeProteinPivotMoveState cState2 =
		 dynamic_cast<const CLatticeProteinPivotMoveState&>(_cState2);
		return (this->seqLength == cState2.seqLength
				  && *this->lattice == *cState2.lattice
				  && *this->energy == *cState2.energy
				  && this->cSequence == cState2.cSequence
				  && this->cStructure == cState2.cStructure);
	}

	bool
	CLatticeProteinPivotMoveState::operator!= (const CState& _cState2) const {
		CLatticeProteinPivotMoveState cState2 =
		 dynamic_cast<const CLatticeProteinPivotMoveState&>(_cState2);
		return (this->seqLength != cState2.seqLength
				  && *this->lattice != *cState2.lattice
				  && *this->energy != *cState2.energy
				  && this->cSequence != cState2.cSequence
				  && this->cStructure != cState2.cStructure);
	}

	CLatticeProteinPivotMoveState* 
	CLatticeProteinPivotMoveState::clone() const {
		return new CLatticeProteinPivotMoveState(*this);
	}

	std::string
	CLatticeProteinPivotMoveState::getCStateStringRepresentation() const {
		std::string strRepr;
		for (size_t i = 0; i < cSequence.size(); i++) {
			strRepr += cSequence[i];
		}
		for (size_t i = 0; i < cStructure.size(); i++) {
			strRepr += cStructure[i];
		}
		return strRepr;
	}

	std::string
	CLatticeProteinPivotMoveState::getUncomprStateStringRepresentation()
	 const {
		return ( getUncompressedStructStr() + "(" +
				  getUncompressedSeqStr() +")" );
	}

	State*
	CLatticeProteinPivotMoveState::uncompressedClone() const {
		return new LatticeProteinPivotMoveState(lattice, energy,
					 getUncompressedSeqStr(), getUncompressedStructStr(), false);
	}

	void
	CLatticeProteinPivotMoveState::uncompress(State& state) const {
		dynamic_cast<LatticeProteinPivotMoveState&>(state) =
		 LatticeProteinPivotMoveState(lattice, energy,
		  getUncompressedSeqStr(), getUncompressedStructStr(), false);
	}

	CLatticeProteinPivotMoveState*
	CLatticeProteinPivotMoveState::readBinaryInputClone(std::ifstream& source)
	 const {

			// create new CLatticeProteinPivotMoveState
		CLatticeProteinPivotMoveState* cState =
		 new CLatticeProteinPivotMoveState();

			// get lattice model and energy function from the dummy state
			// which calls this function
		cState->lattice = this->lattice;
		cState->energy = this->energy;

			// read seqLength
		source.read(reinterpret_cast<char*>(&(cState->seqLength)),
						   sizeof(cState->seqLength));

		size_t size;

			// read cSequence
		source.read(reinterpret_cast<char*>(&size), sizeof(size));
		cState->cSequence.resize(size);
		for (size_t i = 0; i < cState->cSequence.size(); i++) {
			source.read(reinterpret_cast<char*>(&(cState->cSequence[i])),
						  sizeof(cState->cSequence[i]));
		}

			// read cStructure
		source.read(reinterpret_cast<char*>(&size), sizeof(size));
		cState->cStructure.resize(size);
		for (size_t i = 0; i < cState->cStructure.size(); i++) {
			source.read(reinterpret_cast<char*>(&(cState->cStructure[i])),
						  sizeof(cState->cStructure[i]));
		}

		return cState;
	}

	void
	CLatticeProteinPivotMoveState::writeBinaryOutput(std::ofstream& destination)
	 const {
			// write seqLength
		destination.write(
	 	 reinterpret_cast<const char*>(&seqLength), sizeof(seqLength));

		size_t size;

			// write cSequence
		size = cSequence.size();
		destination.write(reinterpret_cast<const char*>(&size), sizeof(size));
		for (size_t i = 0; i < size; i++) {
			destination.write(reinterpret_cast<const char*>(&(cSequence[i])),
							   sizeof(cSequence[i]));
		}

			// write cStructure
		size = cStructure.size();
		destination.write(reinterpret_cast<const char*>(&size), sizeof(size));
		for (size_t i = 0; i < size; i++) {
			destination.write(reinterpret_cast<const char*>(&(cStructure[i])),
							   sizeof(cStructure[i]));
		}
	}

} // namespace ell
