#include <ell/LatticeProtein.hh>
#include <fstream>

namespace ell
{

	LatticeProtein::LatticeProtein( const LatticeModel* _lattice, 
		const ContactEnergyFunction* _energy, const std::string& seqStr, 
		const std::string& moveString, const bool isAbsoluteMove)
	 :	lattice(_lattice), energy(_energy), sequence(NULL), relMoves(NULL),
	 	points(NULL), pointsToUpdate(true)
	{
		assertell (lattice != NULL && energy != NULL, 
			"no lattice model or energy function available");
			// sequence initialisieren
		sequence = new Sequence(energy->getAlphabet()->getSequence(seqStr));
			// relative move sequence initialisieren 
			// je nachdem ob absoluter oder relativer string uebergeben
		if (isAbsoluteMove) {
			relMoves = new MoveSequence(
							lattice->absMovesToRelMoves(
								lattice->parseMoveString(moveString)));
		} else {
			relMoves = new MoveSequence(lattice->parseMoveString(moveString));
		}
		assertell ((relMoves->size()+1) == sequence->size(), 
			"sequence and structure length differ"); // muessen gleich lang sein
	}

	LatticeProtein::LatticeProtein(const LatticeProtein& latPr) 
	 :	lattice(latPr.lattice), energy(latPr.energy), 
	 	sequence(new Alphabet::Sequence(*latPr.sequence)), 
	 	relMoves(new MoveSequence(*(latPr.relMoves))),
	 	points(NULL),pointsToUpdate(true)
	{
		assertell (lattice != NULL && energy != NULL, 
			"no lattice model or energy function available");
		assertell ((relMoves->size()+1) == sequence->size(), 
			"sequence and structure length differ"); // muessen gleich lang sein
	}
	
	LatticeProtein::~LatticeProtein()
	{
		if (sequence != NULL) {
			delete sequence;	sequence = NULL;
		}
		if (relMoves != NULL) {
			delete relMoves;	relMoves = NULL;
		}
		if (points != NULL) {
			delete points;		points = NULL;
		}
	}
	
	IPointVec*
	LatticeProtein::getPoints() const {
		if (points == NULL) {
			assert(sequence != NULL);
			points = new IPointVec(sequence->size());
			pointsToUpdate = true;
		}
		if (pointsToUpdate) {
			lattice->relMovesToPoints( *relMoves, *points );
			pointsToUpdate = false;
		}
		assert(points != NULL);
		return points;
	}

	
	LatticeProtein&
	LatticeProtein::operator =(const LatticeProtein& latProt2) {
		if (this != &latProt2) {
			assertell(sequence != NULL && relMoves != NULL,
				"sequence and structure not initialized");
				// resize sequence if too small
			if (latProt2.sequence->size() != sequence->size()) {
				sequence->resize(latProt2.sequence->size());
				relMoves->resize(latProt2.relMoves->size());
			}
				// copy data
			lattice = latProt2.lattice;
			energy = latProt2.energy;
			std::copy(	latProt2.sequence->begin(),
						latProt2.sequence->end(),
						sequence->begin());
			std::copy(	latProt2.relMoves->begin(),
						latProt2.relMoves->end(),
						relMoves->begin());
			clearPoints();
		}
		return *this;
	}
	
	bool
	LatticeProtein::operator ==(const LatticeProtein& latProt2) const {
		return	*lattice == *(latProt2.lattice)
				&& *energy == *(latProt2.energy)
				&& *sequence == *(latProt2.sequence)
				&& *relMoves == *(latProt2.relMoves);
	}
	
	bool
	LatticeProtein::operator !=(const LatticeProtein& latProt2) const {
		return	*lattice != *(latProt2.lattice)
				|| *energy != *(latProt2.energy)
				|| *sequence != *(latProt2.sequence)
				|| *relMoves != *(latProt2.relMoves);
	}
	
	
	bool 
	LatticeProtein::isSelfAvoiding() const {
		assertell(relMoves != NULL, "no structure available");
		
		IPointVec* pData = getPoints();
	
			// check by set insertion
		IPointSet tmp;
			// pruefe via einfuegen, ob alle punkte unique sind
		for (IPointVec::const_iterator it = pData->begin();  
				it != pData->end(); it++) {
			if (tmp.insert(*it).second == false) // already inside (not inserted)
				return false;	// not selfavoiding
		}

		return true;
	}

	// abstract functions
	DPointVec	
	LatticeProtein::get3Ddata() const  {
		assertell (relMoves != NULL, "no structure available");
		IPointVec* iVec = getPoints();
		DPointVec dVec;
		for (IPointVec::const_iterator it = iVec->begin();it != iVec->end(); it++)
			dVec.push_back((DblPoint)*it);
		return dVec;
	}
	
	double 
	LatticeProtein::getDRMSD(const BackboneStructure3D& other) const {
		DPointVec this3Ddata = get3Ddata(), other3Ddata = other.get3Ddata();

		assertell(this3Ddata.size() == other3Ddata.size(),
		 "Both lattice proteins have to have the same length.");

		double rmsd = 0.0;

		for(size_t i=0; i < this3Ddata.size()-1; i++) {
			for(size_t j=i+1; j < this3Ddata.size(); j++) {
				rmsd += std::pow( this3Ddata[i].distance(this3Ddata[j])
						 - other3Ddata[i].distance(other3Ddata[j]), 2 );
			}
		}

        rmsd /= (this3Ddata.size() * (this3Ddata.size()-1) /2);

        return std::sqrt(rmsd);

	}

	double	
	LatticeProtein::getEnergy() const {
		assertell(lattice != NULL, "no lattice model available");
		double retVal = 0.0;
		IPointVec* pData = getPoints();
		IPointVec::size_type i=0,j=0;
		for (i=0; i < pData->size(); i++) {
			for (j = i+2; j < pData->size(); j++) {
				if (lattice->areNeighbored(pData->at(i), pData->at(j))) {
					retVal += energy->getContactEnergy(	sequence->at(i),
														sequence->at(j));
				}
			}
		}
		return retVal;
	}

	bool
	LatticeProtein::writeXYZfile(std::string & filename) const {
		assertell(filename.size() != 0, "filename not declared to write XYZ file");
		bool allDone = true;
		if (filename.size() < 5 
			&& filename.compare(filename.size()-4,4,".xyz") != 0
			&& filename.compare(filename.size()-4,4,".XYZ") != 0 )
		{
			filename += ".xyz";
		}
		
		std::ofstream fout(filename.c_str(), std::ios::trunc );
		allDone = fout.good();
		
		if (allDone) {
			DPointVec points = get3Ddata();
			fout	<<"# ELL - lattice protein\n"
					<<"# lattice model  : " <<lattice->getDescriptor()->getName() <<"\n"
					<<"# absolute moves : " <<getAbsoluteMoveString() <<std::endl;
			for (DPointVec::const_iterator p = points.begin(); p!=points.end(); p++) {
				fout <<p->getX() <<" " <<p->getY() <<" " <<p->getZ() <<std::endl;
			}
			fout	<<"# EOF #" <<std::endl;
			fout.flush();
		}
		fout.close();			
		
		return allDone;
	}


}	// namespace ell
