#include <ell/RNAFreeEnergy.hh>
#include <ell/assertell.hh>

#include <ViennaRNA/fold_vars.h>
extern "C"	float energy_of_struct(const char *string, const char *structure);


#include <iostream>
#include <cstring>  //For the string functions

namespace ell
{

	const Alphabet RNAFreeEnergy::COMMON_ALPHABET("ACGU", 1);
	const AllowedBasePairs RNAFreeEnergy::COMMON_BPAIRS(
							&COMMON_ALPHABET, "AU,CG,GU");
//////////////////////////////////////////////////////////
// RNAFreeEnergy
//////////////////////////////////////////////////////////

	RNAFreeEnergy::RNAFreeEnergy(const std::string& rnaSeqStr,
	 							 const std::string& rnaStructBracketDotStr)
	 :	RNAStructure(rnaSeqStr, rnaStructBracketDotStr, &COMMON_BPAIRS),
		seqStr(NULL), structStr(NULL)
	{
		// ViennaRNA package parameter for dangling ends
		dangles = 2;
		
		seqStr = new char[rnaSeqStr.size()];
		strcpy(seqStr,rnaSeqStr.c_str());
		structStr = new char[rnaStructBracketDotStr.size()];
		strcpy(structStr, rnaStructBracketDotStr.c_str());
	}
	
	RNAFreeEnergy::RNAFreeEnergy(const RNAFreeEnergy& rnaFreeEnerg)
	 : RNAStructure(rnaFreeEnerg), seqStr(NULL),
		structStr(NULL)
	{
		seqStr = new char[rnaFreeEnerg.rnaSeq->size()];
		strcpy(seqStr,rnaFreeEnerg.seqStr);
		structStr = new char[rnaFreeEnerg.rnaStructBracketDot->size()];
		strcpy(structStr, rnaFreeEnerg.structStr);
	}

	RNAFreeEnergy::~RNAFreeEnergy()	{
		if (seqStr != NULL) delete [] seqStr;
		if (structStr != NULL) delete [] structStr;
	}

	size_t	
	RNAFreeEnergy::nextCompatibleSingleMovePos(const size_t pos) const {
		for (	size_t p = pos+1; 
				p < rnaLength && (*rnaStructBracketDot)[p] != STRUCT_BND_CLOSE; 
				(p = rnaBonds[p])++) 
		{
			if ((*rnaStructBracketDot)[p] == STRUCT_UNBOUND) {
				return p;
			}
		}
		return RNAStructure::INVALID_INDEX;
	}

	void		
	RNAFreeEnergy::applySingleMoveInPlace(const SingleMove& move) {
		// open bond between move.first and move.second
		if (rnaBonds[move.first] == move.second) {
			(*rnaStructBracketDot)[move.first] = STRUCT_UNBOUND;
			structStr[move.first] = '.';
			(*rnaStructBracketDot)[move.second] = STRUCT_UNBOUND;
			structStr[move.second] = '.';
			rnaBonds[move.first] = RNAStructure::INVALID_INDEX;
		}
		// close bond between pos1 and pos2
		else {
			assertell(isValidSingleMove(move),
				"can't close bond with single move, because it is not valid");
			(*rnaStructBracketDot)[move.first] = STRUCT_BND_OPEN;
			structStr[move.first] = '(';
			(*rnaStructBracketDot)[move.second] = STRUCT_BND_CLOSE;
			structStr[move.second] = ')';
			rnaBonds[move.first] = move.second;
		}
	}

	bool		
	RNAFreeEnergy::isValidSingleMove(const SingleMove& move)
			 const {
			 	
		assertell(move.first < move.second && move.second < rnaLength,
				"first move pos has to be smaller than second");
			 	
		// bond between move.first and move.second
		if (rnaBonds[move.first] == move.second) {
			return true;
		}

		// check if both unpaired and not destroying nestedness
		if (	(*rnaStructBracketDot)[move.first] == STRUCT_UNBOUND &&
				(*rnaStructBracketDot)[move.second] == STRUCT_UNBOUND &&
					// distance between move.first and move.second has to be 
					// at least RNAStructure::MIN_LOOP_LENGTH
				(move.second-move.first) > RNAStructure::MIN_LOOP_LENGTH ) 
		{
			size_t i = move.first; 
			while (i < move.second ) {
				if((*rnaStructBracketDot)[i] == STRUCT_BND_CLOSE) {
					return false;
				} else if((*rnaStructBracketDot)[i] == STRUCT_BND_OPEN) {
					i = rnaBonds[i]; 
					i++;
				} else {
					i++; 
				}
			}
			return i==move.second && isAllowedBasePair(move.first, move.second);
		}
		
		// else no valid structure
		return false;
	}

	double	
	RNAFreeEnergy::getEnergy() const {
		return energy_of_struct( seqStr, structStr );
	}


	RNAFreeEnergy& 	
	RNAFreeEnergy::operator= (const RNAFreeEnergy& rnaStruct2) {
		
		if (this != &rnaStruct2) {
			if (rnaSeq->size() != rnaStruct2.rnaSeq->size()) {
				if (seqStr != NULL) delete [] seqStr;
				if (structStr != NULL) delete [] structStr;
				seqStr = new char[rnaStruct2.rnaSeq->size()];
				structStr = new char[rnaStruct2.rnaStructBracketDot->size()];
				rnaSeq->resize(rnaStruct2.rnaSeq->size());
				rnaStructBracketDot->resize(rnaStruct2.rnaStructBracketDot->size());
			}
			strcpy(seqStr, rnaStruct2.seqStr);
			strcpy(structStr, rnaStruct2.structStr);
			std::copy(	rnaStruct2.rnaSeq->begin(),
						rnaStruct2.rnaSeq->end(),
						rnaSeq->begin());
			std::copy(	rnaStruct2.rnaStructBracketDot->begin(),
						rnaStruct2.rnaStructBracketDot->end(),
						rnaStructBracketDot->begin());
			bPair = rnaStruct2.bPair;
			rnaLength = rnaStruct2.rnaLength;	
			rnaBonds = rnaStruct2.rnaBonds;
		}
		return *this;
	}


	
} // namespace ell
