#ifndef RNAFREEENERGYSINGLEMOVESTATE_HH_
#define RNAFREEENERGYSINGLEMOVESTATE_HH_


#include "State.hh"
#include "RNAFreeEnergy.hh"

namespace ell
{
		/**
		 * This class generates neighbors in the energy landscape by
		 * adding or removing one bond to the RNA structure.
		 * 
		 * @author Martin Mann, Sebastian Will, Andreas Richter
		 */
	class RNAFreeEnergySingleMoveState : public State
	{
	protected:	
		typedef RNAFreeEnergy::SingleMove	SingleMove;

		class NeighborList : public State::NeighborList
		{
		protected:
				//! the state of origin of the neighbors
			const RNAFreeEnergySingleMoveState* const origin;

		public:
				//! State of an iterator. Usually the information in ItState is
				//! used for computing the next list entry 
			class ItState : public State::NeighborList::ItState
			{
			public:
					//! the last applied single move
				SingleMove	lastMove;

				ItState() : lastMove() {}
				ItState(const ItState &itstate)
				 : lastMove(itstate.lastMove) {}
				virtual ~ItState() {}

				virtual ItState* clone() const {
					return new ItState(*this);
				}
			};

			NeighborList(const RNAFreeEnergySingleMoveState* _origin);
			
			virtual ~NeighborList();

				/*! Returns a pointer to the first element of the
				 *  virtual list of neighbors of the state of origin.
				 *  Returns NULL if no neighbors exist.
				 */
			virtual State* first(State::NeighborList::ItState** itstate) const;
						 
				/*! Returns a pointer to the next element of the
				 *  virtual list of neighbors of the state of origin.
				 *  Returns NULL if no more neighbors exist.
				 */
			virtual State* next(	State::NeighborList::ItState* itstate, 
									State* elem) const;
		};

		class RandomNeighborList : public State::NeighborList
		{
		public:
			class ItState;
			
		protected:
				//! the state of origin of the neighbors
			const RNAFreeEnergySingleMoveState* const origin;

				//! Changes the mode of choice of the next single move from
				//! choice out of all single moves to choice out of all
				//! unchosen valid single moves
			void	switchMode(ItState* itstate) const;
			
		public:
			class ItState : public State::NeighborList::ItState
			{
			public:
					//! if the numbers of valid single moves that have been
					//! selected more than once exceeds this threshold, the
					//! mode of choice will be changed to choice out of all
					//! unchosen valid moves
				static const size_t SWITCH_MODE_THRESHOLD;
				
					//! counts down from SWITCH_MODE_THRESHOLD to zero for
					//! every valid move chosen more than once
				size_t switchModeValue;
				
					//! set containing all valid single moves that were randomly
					//! chosen during iteration
				std::set<SingleMove> chosenMoves;
				
					//! vector containing all VALID single moves which have
					//! not been chosen yet
				std::vector<SingleMove> unchosenValidMoves;

				ItState() : switchModeValue(SWITCH_MODE_THRESHOLD),
							chosenMoves(),
							unchosenValidMoves()
				{}
				
				ItState(const ItState &itstate) 
					: switchModeValue(itstate.switchModeValue),
					  chosenMoves(itstate.chosenMoves),
					  unchosenValidMoves(itstate.unchosenValidMoves)
				{}
				
				virtual ~ItState() 
				{}

				virtual ItState* clone() const {
					return new ItState(*this);
				}
			};

			RandomNeighborList(const RNAFreeEnergySingleMoveState* _origin);
			
			virtual ~RandomNeighborList();

				/*! Returns a pointer to a random element of the
				 *  virtual list of neighbors of the state of origin.
				 *  Returns NULL if no neighbors exist.
				 */
			virtual State* first(State::NeighborList::ItState** itstate) const;
						 
				/*! Returns a pointer to the next random element of the
				 *  virtual list of neighbors of the state of origin.
				 *  Returns NULL if no more neighbors exist.
				 */
			virtual State* next(	State::NeighborList::ItState* itstate, 
									State* elem) const;
		};

		RNAFreeEnergy rnaFreeEnergy;

		ell::Alphabet::AlphElem open;
		ell::Alphabet::AlphElem none;
		ell::Alphabet::AlphElem close;

			/*! Returns the first VALID single move.
			 *  Returns SingleMove(0,RNAStructure::INVALID_INDEX)
			 *  if no first valid single move exits.
			 */
		SingleMove	firstValidSingleMove() const;
		
			/*! Returns the next VALID single move if lastSingleMove
			 *  was the last single move.
			 *  Returns SingleMove(0,RNAStructure::INVALID_INDEX)
			 *  if no next valid single move exits.
			 *
			 *  lastSingleMove has to be an valid move!
			 */
		SingleMove nextValidSingleMove(const SingleMove lastSingleMove) const;
		
			/*! Searches for the next valid single move and starts
			 *  the search for the first move position at pos 1
			 *  and for the second move position at pos2.
			 *  If a move was found, it will be returned, otherwise
			 *  SingleMove(0,RNAStructure::INVALID_INDEX) will be returned.
			 */
		SingleMove searchForNextValidSingleMove(size_t pos1,
				 							size_t pos2) const;
	
	public:
		RNAFreeEnergySingleMoveState(const std::string& rnaSeqStr,
									 const std::string& rnaStructBracketNotStr);
				
		RNAFreeEnergySingleMoveState( 
						const RNAFreeEnergySingleMoveState& rnaFESMS);
						
		virtual ~RNAFreeEnergySingleMoveState();

	// abstract functions (State)
		virtual bool	operator== (const State& state2) const;
		virtual bool	operator!= (const State& state2) const;

			/*! Returns the negatived free energy of this RNA state multiplied with 100.
			 */
		virtual int	getFitness() const;
		
			/*! Returns the state specific energy. */
		virtual double		getEnergy() const;
		
			/*! Returns the minimal number of steps via valid
			 *  neighbored states from this to another valid State.
			 *  @param state2 the State to reach
			 */
		virtual unsigned int	getMinimalDistance(const State& state2) const;
		
			/*! Returns a pointer to a clone of the current state. */
		virtual RNAFreeEnergySingleMoveState*	clone() const;
		
			/*! Returns a specific std::string representation of this
			 *  State.
			 * 
			 *  BRACKETNOTATION(SEQUENCE) */
		virtual std::string getStateStringRepresentation() const;

		virtual CState* compressedClone() const;

		virtual void	compress(CState& cState) const;

		virtual NeighborListPtr getNeighborList() const;
		
		virtual NeighborListPtr getRandomNeighborList(State* inPlaceNeigh = NULL) const;
		
		virtual State*	getRandomNeighbor(State* inPlaceNeigh = NULL) const;

	};

} // namespace ell

#endif /*RNAFREEENERGYSINGLEMOVESTATE_HH_*/
